#!/bin/bash
#==============================================================================
# Copyright and license info is available in the LICENSE file included with
# the Helix Management System (HMS), and also available online:
# https://swarm.workshop.perforce.com/projects/perforce-software-hms/view/main/LICENSE
#------------------------------------------------------------------------------

#==============================================================================
# Declarations and Environment

# Allow override of P4U_HOME, which is set only when testing P4U scripts.
export P4U_HOME="${P4U_HOME:-/p4/common/bin}"
export P4U_LIB="${P4U_LIB:-/p4/common/lib}"
export P4U_ENV="$P4U_LIB/p4u_env.sh"
export TOOLS_DIR="${TOOLS_DIR:-.}"
export P4U_LOG="Unset"
export VERBOSITY="${VERBOSITY:-3}"

# Environment isolation.  For stability and security reasons, prepend
# PATH to include dirs where known-good scripts exist.
# known/tested PATH and, by implication, executables on the PATH.
export PATH=$P4U_HOME:$PATH:~/bin:.
export P4CONFIG=${P4CONFIG:-.p4config}

[[ -r "$P4U_ENV" ]] || {
   echo -e "\\nError: Cannot load environment from: $P4U_ENV\\n\\n"
   exit 1
}

declare BASH_LIBS=$P4U_ENV
BASH_LIBS+=" $P4U_LIB/libcore.sh"
BASH_LIBS+=" $P4U_LIB/libp4u.sh"
BASH_LIBS+=" $TOOLS_DIR/env.sh"

for bash_lib in $BASH_LIBS; do
   source $bash_lib ||\
      { echo -e "\\nFATAL: Failed to load bash lib [$bash_lib]. Aborting.\\n"; exit 1; }
done

declare Version="1.2.0"
declare ReportStyle="OpenJobs"
declare ReportStyleTitle=
declare JobSearchQuery=
declare WorkshopUser="Unset"
declare JobStatusValues=
declare MaxTitleLength=70
declare -i ShowUserJobsOnly=0
declare -i SilentMode=0
declare -i JobCount=0
export VERBOSITY=3

#==============================================================================
# Local Functions

#------------------------------------------------------------------------------
# Function: terminate
function terminate
{
   # Disable signal trapping.
   trap - EXIT SIGINT SIGTERM

   vvmsg "$THISSCRIPT: EXITCODE: $OverallReturnStatus"

   # Stop logging.
   [[ "${P4U_LOG}" == off ]] || stoplog

   # Don't litter.
   cleanTrash

   # With the trap removed, exit.
   exit $OverallReturnStatus
}

#------------------------------------------------------------------------------
# Function: usage (required function)
#
# Input:
# $1 - style, either -h (for short form) or -man (for man-page like format).
# The default is -h.
#
# $2 - error message (optional).  Specify this if usage() is called due to
# user error, in which case the given message displayed first, followed by the
# standard usage message (short or long depending on $1).  If displaying an
# errror, usually $1 should be -h so that the longer usage message doesn't
# obsure the error message.
#
# Sample Usage:
# usage
# usage -h
# usage -man
# usage -h "Incorrect command line usage."
#------------------------------------------------------------------------------
function usage
{
   declare style=${1:--h}
   declare errorMessage=${2:-Unset}

   if [[ $errorMessage != Unset ]]; then
      echo -e "\\n\\nUsage Error:\\n\\n$errorMessage\\n\\n"
   fi

   echo "USAGE for $THISSCRIPT v$Version:

$THISSCRIPT [-a|-o] [-me | -u <workshop_user>] [-st <status1>[,<status2>,...]] [-L <log>] [-si] [-v<n>] [-n] [-D]

or

$THISSCRIPT [-h|-man|-V]
"
   if [[ $style == -man ]]; then
      echo -e "
DESCRIPTION:
	This script procduces HMS jobs reports.

OPTIONS:
 -a	Show all SDP jobs regardless of the state.

 -o	Show only "Open" SDP jobs, which includes jobs in these states:
        * open
        * inprogress
        * blocked

	This skips jobs in these states:
	* closed
        * duplicate
        * fixed (optional status to use before closed).
	* punted
        * obsolete
	* suspended

	The '-o' behavior is the default.

 -me	Show only jobs for which the OwnedBy setting is the current user.
	Guessing logic is applied to map the current OS user to a Workshop account,
	e.g. ttyler -> tom_tyler, etc.  This works for some SDP project members.

	See the guess_workshop_user() function in env.sh for details.

	Works with '-a' and '-o'.

 -u <workshop_user>
	Show only jobs for which the OwnedBy setting is the specified Workshop user
	account.

	Specify the special value 'none' to list unassigned jobs, i.e. those for
	which the OwnedBy field is not set.

	Works with '-a' and '-o'.

 -st <status1>[,<status2>,...]
	Specify a comma-delimited list of status values to include.

	The valid values can be seen by doing: p4 jobspec -o

 -v<n>	Set verbosity 1-5 (-v1 = quiet, -v5 = highest).

	The default is -v3.

	Specify -v4 to see the query used with the 'p4 jobs -e' command.

 -L <log>
	Specify the path to report log file, or the special value 'off' to disable
	logging.  By default, all output (stdout and stderr) goes to a file, then
	name of which is displayed when the script starts.

	NOTE: This script is self-logging.  That is, output displayed on the screen
	is simultaneously captured in the log file.  Do not run this script with
	redirection operators like '> log' or '2>&1', and do not use 'tee.'

-si	Operate silently.  All output (stdout and stderr) is redirected to the log
	only; no output appears on the terminal.  This cannot be used with
	'-L off'.
      
 -n	No-Op.  Prints commands instead of running them.

 -D     Set extreme debugging verbosity.

HELP OPTIONS:
 -h	Display short help message
 -man	Display man-style help message
 -V	Dispay version info for this script and its libraries.

EXAMPLES:
	Typical usage is with no arguments, to display open jobs:

	$THISSCRIPT

	Show opened jobs assigned to me (based on the \$WORKSHOP_USER shell
	environment setting).

	$THISSCRIPT -me

	Show only items listed as inprogress:

	$THISSCRIPT -st inprogress

SEE ALSO:
"
   fi

   exit 1
}

#==============================================================================
# Command Line Processing

declare -i shiftArgs=0

set +u
while [[ $# -gt 0 ]]; do
   case $1 in
      (-a) ReportStyle="AllJobs";;
      (-o) ReportStyle="OpenJobs";;
      (-me) ShowUserJobsOnly=1; WorkshopUser="me";;
      (-u) ShowUserJobsOnly=1; WorkshopUser="$2"; shiftArgs=1;;
      (-st) JobStatusValues="$2"; shiftArgs=1;;
      (-h) usage -h;;
      (-man) usage -man;;
      (-V) show_versions; exit 1;;
      (-v1) export VERBOSITY=1;;
      (-v2) export VERBOSITY=2;;
      (-v3) export VERBOSITY=3;;
      (-v4) export VERBOSITY=4;;
      (-v5) export VERBOSITY=5;;
      (-L) export P4U_LOG="$2"; shiftArgs=1;;
      (-si) SilentMode=1;;
      (-n) export NO_OP=1;;
      (-D) set -x;; # Debug; use 'set -x' mode.
      (*) usage -h "Unknown arg ($1).";;
   esac

   # Shift (modify $#) the appropriate number of times.
   shift; while [[ $shiftArgs -gt 0 ]]; do
      [[ $# -eq 0 ]] && usage -h "Incorrect number of arguments."
      shiftArgs=$shiftArgs-1
      shift
   done
done
set -u

#==============================================================================
# Command Line Verification

[[ $SilentMode -eq 1 && "$P4U_LOG" == off ]] && \
   usage -h "Cannot use '-si' with '-L off'."

[[ "$P4U_LOG" == "Unset" ]] && export P4U_LOG="${LOGS:-.}/jobs_report.${WORKSHOP_PROJECT_TAG}.$(date +'%Y%m%d-%H%M').txt"

[[ "$WorkshopUser" == "me" ]] && WorkshopUser="$(guess_workshop_user)"

#==============================================================================
# Main Program

trap terminate EXIT SIGINT SIGTERM

declare -i OverallReturnStatus=0

if [[ "${P4U_LOG}" != off ]]; then
   touch ${P4U_LOG} || bail "Couldn't touch log file [${P4U_LOG}]."

   # Redirect stdout and stderr to a log file.
   if [[ $SilentMode -eq 0 ]]; then
      exec > >(tee ${P4U_LOG})
      exec 2>&1
   else
      exec >${P4U_LOG}
      exec 2>&1
   fi

   initlog
fi

if [[ $ReportStyle == "OpenJobs" ]]; then
   ReportStyleTitle="Open Jobs"

   if [[ -n "$JobStatusValues" ]]; then
      # Query to show only SDP jobs in specific states.
      if [[ "$JobStatusValues" =~ /,/ ]];then
         # Query for a single job status
         JobSearchQuery="Project=$WORKSHOP_PROJECT Status=$JobStatusValues"
      else
         # Query for multiple job status values
         JobSearchQuery="Project=$WORKSHOP_PROJECT ("
         declare -i first=1
         for status in ${JobStatusValues/,/ }; do
            if [[ $first -eq 1 ]]; then
               first=0
               JobSearchQuery+="Status=$status"
            else
               JobSearchQuery+="|Status=$status"
            fi
         done
         JobSearchQuery+=")"
      fi
   else
      # Query to show only "Open" SDP jobs. See usage info for this script and
      # 'p4 jobspec -o' for details on all states.
      JobSearchQuery="Project=$WORKSHOP_PROJECT ^Status=closed ^Status=duplicate ^Status=fixed ^Status=obsolete ^Status=punted ^Status=suspended"
   fi
else
   ReportStyleTitle="Open Jobs"

   # Query to show all SDP jobs.
   JobSearchQuery="Project=$WORKSHOP_PROJECT"
fi

[[ "$ShowUserJobsOnly" -eq 1 ]] && ReportStyleTitle+=" owned by $WorkshopUser"

vmsg "Job Search Query: $JobSearchQuery"

printf "SDP Jobs Report showing $ReportStyleTitle\\n\\n %-8s %-12s %-2s %-16s %-12s %-s\\n" "Job" "Status" "Pr" "Owner" "Component" "Title"
printf " %-8s %-12s %-2s %-16s %-12s %-s\\n" "--------" "------------" "--" "----------------" "------------" "----------------------------------------------------------------------"

for job in $(p4 -ztag -F %Job% jobs -e "$JobSearchQuery"); do
   jobData=$(p4 -ztag -F "%OwnedBy%:%Component%:%Status%:%Severity%" job -o $job)
   ownedBy="${jobData%%:*}"
   status="${jobData#*:}"
   status="${status#*:}"
   status="${status%:*}"
   component="${jobData#*:}"
   component="${component%%:*}"
   priority=${jobData##*:}

   # Skip jobs by users other than the current user of '-me' was specified.
   if [[ $ShowUserJobsOnly -eq 1 ]]; then
      if [[ "$WorkshopUser" == "none" ]]; then
         [[ -n "$ownedBy" ]] && continue
      else
         [[ "$ownedBy" != "$WorkshopUser" ]] && continue
      fi
   fi

   jobTitle=$(p4 -ztag -F %Description% job -o $job | head -1)
   jobTitle=$(echo $jobTitle | cut -c -$MaxTitleLength)
   printf " %-8s %-12s %-2s %-16s %-12s %-s\\n" "$job" "$status" "$priority" "$ownedBy" "$component" "$jobTitle"
   JobCount+=1
done

msg "${H}\\n$JobCount jobs reported in $(($SECONDS/3600)) hours $(($SECONDS%3600/60)) minutes $(($SECONDS%60)) seconds.\n"

# See the terminate() function, which is really where this script exits.
exit $OverallReturnStatus
